import argparse
import logging
import os
import sys
import urllib.parse

import yaml
import zmq


def manual_run(request, cfg):
    """
    Run the report service manually from any machine
    and provide the requested configuration for
    reports generation.

    :param request: a list for generating reports for the
                     requested Instruments. This list can
                     contain the Instruments names e.g ['SPB']
                     or ['all'] for generating reports for all
                     instruments in the "report_conf.yaml".

                     This can also be a customized conf. file(dict)
                     for test purposes.
    """

    port = cfg['GLOBAL']['server-port']
    con = zmq.Context()
    socket = con.socket(zmq.REQ)
    con = socket.connect(port)
    socket.send_pyobj(request)
    msg = socket.recv_pyobj()
    logging.info('{} Manual Run'.format(msg))

arg_parser = argparse.ArgumentParser(description='Manual Launch')
arg_parser.add_argument('--instrument', default=['all'], nargs='+',
                        help='select the requested instruments. '
                             'Default=all')
arg_parser.add_argument('--testing', dest='testing', action='store_true',
                        help='required for testing with different '
                             'config files')
arg_parser.set_defaults(testing=False)
arg_parser.add_argument('--config-file', type=str, 
                        default='./report_conf.yaml',
                        help='config file path with reportservice port. '
                             'Default=./report_conf.yaml')
arg_parser.add_argument('--log-file', type=str, default='./report.log',
                        help='The report log file path. Default=./report.log')
arg_parser.add_argument('--logging', type=str, default="INFO", 
                        help='logging modes: INFO, DEBUG or ERROR. '
                             'Default=INFO',
                        choices=['INFO', 'DEBUG', 'ERROR'])

if __name__ == "__main__":
    args = vars(arg_parser.parse_args())
    conf_file = args["config_file"]
    with open(conf_file, "r") as f:
        cfg = yaml.load(f.read())

    logfile = args["log_file"]
    fmt = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'

    logging.basicConfig(filename=logfile, filemode='a+',
                        level=getattr(logging, args['logging']),
                        format='%(levelname)-6s: %(asctime)s %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S')
    if args["testing"]:
        request = cfg
    else: 
        request = args["instrument"]

    manual_run(request, cfg)
